@@ -44,6 +44,7 @@
 
 #include "gthumb-init.h"
 #include "gthumb-error.h"
+#include "glib-utils.h"
 #include "gconf-utils.h"
 #include "file-utils.h"
 
@@ -336,7 +337,7 @@ path_list_find_path (GList *list, const char *path)
 	GList *scan;
 
 	for (scan = list; scan; scan = scan->next)
-		if (strcmp ((char*) scan->data, path) == 0)
+		if (same_uri ((char*) scan->data, path))
 			return scan;
 	return NULL;
 }
@@ -613,6 +614,29 @@ get_sample_name (const char *filename)
 /* File utils */ 
 
 
+const char *
+get_mime_type (const char *uri)
+{
+	if (uri_scheme_is_file (uri))
+		uri = get_file_path_from_uri (uri);
+	return gnome_vfs_get_file_mime_type (uri, NULL, FALSE);
+}
+
+
+const char *
+get_mime_type_from_ext (const char *ext)
+{
+	char       *filename;
+	const char *result;
+
+	filename = g_strconcat ("x.", ext, NULL);
+	result = get_mime_type (filename);
+	g_free (filename);
+
+	return result;
+}
+
+
 gboolean
 file_is_image (const gchar *name,
 	       gboolean     fast_file_type)
@@ -635,13 +659,16 @@ file_is_image (const gchar *name,
 
 			if (n3 != NULL)
 				result = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
-
+			
 			g_free (n3);
 			g_free (n2);
 			g_free (n1);
 		}
-	} else 
+	} else {
+		if (uri_scheme_is_file (name))
+			name = get_file_path_from_uri (name);
 		result = gnome_vfs_get_file_mime_type (name, NULL, FALSE);
+	}
 
 	/* Unknown file type. */
 	if (result == NULL)
@@ -675,7 +702,7 @@ xfer_file (const char *from,
 	GnomeVFSXferOptions  opt;
 	GnomeVFSResult       result;
 
-	if (strcmp (from, to) == 0) {
+	if (same_uri (from, to)) {
 		g_warning ("cannot copy file %s: source and destination are the same\n", from);
 		return FALSE;
 	}
@@ -728,10 +755,9 @@ file_unlink (const gchar *path)
 }
 
 
-static gboolean 
-image_is_type__common (const char *name,
-		       const char *type,
-		       gboolean    fast_file_type)
+const char*
+get_file_mime_type (const char *name,
+		    gboolean    fast_file_type)
 {
 	const char *result = NULL;
 
@@ -749,11 +775,17 @@ image_is_type__common (const char *name,
 	} else 
 		result = gnome_vfs_get_file_mime_type (name, NULL, FALSE);
 	
-	/* Unknown file type. */
-	if (result == NULL)
-		return FALSE;
+	return result;
+}
 
-	return (strcmp (result, type) == 0);
+
+static gboolean 
+image_is_type__common (const char *name, 
+		       const char *type,
+		       gboolean    fast_file_type)
+{
+	const char *result = get_file_mime_type (name, fast_file_type);  
+	return (strcmp_null_tollerant (result, type) == 0);
 }
 
 
@@ -1054,9 +1086,6 @@ uri_scheme_is_file (const char *uri)
 {
 	if (uri == NULL)
 		return FALSE;
-	/*
-	return ! uri_scheme_is_catalog (uri) && ! uri_scheme_is_search (uri);
-	*/
 	if (g_utf8_strlen (uri, -1) < FILE_PREFIX_L)
 		return FALSE;
 	return strncmp (uri, FILE_PREFIX, FILE_PREFIX_L) == 0;
@@ -1199,6 +1228,33 @@ path_in_path (const char  *path_src,
 }
 
 
+int
+uricmp (const char *path1,
+	const char *path2)
+{
+	char *uri1, *uri2;
+	int   result;
+
+	uri1 = get_uri_from_path (path1);
+	uri2 = get_uri_from_path (path2);
+
+	result = strcmp_null_tollerant (uri1, uri2);
+
+	g_free (uri1);
+	g_free (uri2);
+
+	return result;
+}
+
+
+gboolean
+same_uri (const char *uri1,
+	  const char *uri2)
+{
+	return uricmp (uri1, uri2) == 0;
+}
+
+
 /* example 1 : filename = /xxx/yyy/zzz/foo 
  *             destdir  = /xxx/www 
  *             return   : ../yyy/zzz/foo
