@@ -923,8 +923,11 @@ get_image_filename (CatalogWebExporter *ce,
 					(location ? "/" : ""),
 					file_name_from_path (idata->dest_filename),
 					NULL);
-	else
-		filename = g_strdup (idata->src_filename);
+	else {
+		filename = gnome_vfs_get_uri_from_local_path (idata->src_filename);
+		if (filename == NULL)
+			filename = g_strdup (idata->src_filename);
+	}
 	
 	return filename;
 }
@@ -1422,7 +1425,7 @@ gth_parsed_doc_print (GList              *document,
 			write_markup_escape_line (line, fout);
 			g_free (line);
 
-			write_line (" ", fout);
+			write_line (" &nbsp; ", fout);
 
 			line = get_exif_tag (idata->src_filename, 
 					     EXIF_TAG_MODEL);
@@ -1618,6 +1621,14 @@ copy_exif_from_orig (const char *src_filename,
 	JPEGData     *jdata_src, *jdata_dest;
 	ExifData     *edata_src;
 
+	src_filename = get_file_path_from_uri (src_filename);
+	if (src_filename == NULL)
+		return;
+
+	dest_filename = get_file_path_from_uri (dest_filename);
+	if (dest_filename == NULL)
+		return;
+
 	jdata_src = jpeg_data_new_from_file (src_filename);
 	if (jdata_src == NULL)
 		return;
