@@ -24,25 +24,24 @@
 #include <string.h>
 #include <time.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-dialog-util.h>
+#include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-dateedit.h>
-#include <libgnomeui/gnome-file-entry.h>
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs.h>
+
+#include "file-utils.h"
 #include "catalog.h"
 #include "comments.h"
 #include "dlg-search.h"
 #include "gthumb-init.h"
 #include "gconf-utils.h"
 #include "search.h"
-#include "gthumb-window.h"
+#include "gth-browser.h"
 #include "gtk-utils.h"
 #include "glib-utils.h"
-#include "utf8-fnmatch.h"
 
 
 enum {
@@ -61,32 +60,31 @@ enum {
 #define CATEGORY_SEPARATOR_STR ";"
 #define ONE_DAY (60*60*24)
 
-static void dlg_search_ui (GThumbWindow *window, 
-			   char         *catalog_path, 
-			   gboolean      start_search);
+static void dlg_search_ui (GthBrowser *browser, 
+			   char       *catalog_path, 
+			   gboolean    start_search);
 
 
 void 
 dlg_search (GtkWidget *widget, gpointer data)
 {
-	GThumbWindow *window = data;
-	dlg_search_ui (window, NULL, FALSE);
+	dlg_search_ui (GTH_BROWSER (data), NULL, FALSE);
 }
 
 
 void 
-dlg_catalog_edit_search (GThumbWindow *window,
-			 const char   *catalog_path)
+dlg_catalog_edit_search (GthBrowser *browser,
+			 const char *catalog_path)
 {
-	dlg_search_ui (window, g_strdup (catalog_path), FALSE);
+	dlg_search_ui (browser, g_strdup (catalog_path), FALSE);
 }
 
 
 void 
-dlg_catalog_search (GThumbWindow *window,
-		    const char   *catalog_path)
+dlg_catalog_search (GthBrowser *browser,
+		    const char *catalog_path)
 {
-	dlg_search_ui (window, g_strdup (catalog_path), TRUE);
+	dlg_search_ui (browser, g_strdup (catalog_path), TRUE);
 }
 
 
@@ -96,14 +94,13 @@ dlg_catalog_search (GThumbWindow *window,
 #define SEARCH_GLADE_FILE      "gthumb_search.glade"
 
 typedef struct {
-	GThumbWindow   *window;
+	GthBrowser     *browser;
 	GladeXML       *gui;
 
 	GtkWidget      *dialog;
 	GtkWidget      *search_progress_dialog;
 
-	GtkWidget      *s_start_from_fileentry;
-	GtkWidget      *s_start_from_entry;
+	GtkWidget      *s_start_from_filechooserbutton;
 	GtkWidget      *s_include_subfold_checkbutton;
 	GtkWidget      *s_filename_entry;
 	GtkWidget      *s_comment_entry;
@@ -220,27 +217,48 @@ destroy_cb (GtkWidget  *widget,
 static void search_images_async (DialogData *data);
 
 
+static void
+start_search_now (DialogData *data) 
+{
+	/* pop up the progress dialog. */
+
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
+	gtk_widget_hide (data->dialog);
+
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (data->p_notebook), 0);
+	gtk_widget_set_sensitive (data->p_searching_in_hbox, TRUE);
+	gtk_widget_set_sensitive (data->p_view_button, FALSE);
+	gtk_widget_set_sensitive (data->p_search_button, FALSE);
+	gtk_widget_set_sensitive (data->p_close_button, FALSE);
+	gtk_label_set_text (GTK_LABEL (data->p_images_label), "");
+
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
+	gtk_widget_show (data->search_progress_dialog);
+
+	search_images_async (data);
+}
+
+
 /* called when the "search" button is pressed. */
 static void
 search_clicked_cb (GtkWidget  *widget, 
 		   DialogData *data)
 {
 	char       *full_path;
+	char       *esc_path;
 	const char *entry;
-	char       *utf8_path;
 
 	/* collect search data. */
 
 	free_search_criteria_data (data);
 	search_data_free (data->search_data);
-
 	data->search_data = search_data_new ();
 
 	/* * start from */
 
-	utf8_path = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (data->s_start_from_fileentry), FALSE);
-	full_path = g_filename_from_utf8 (utf8_path, -1, 0, 0, 0);
-	g_free (utf8_path);
+	esc_path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (data->s_start_from_filechooserbutton));
+	full_path = gnome_vfs_unescape_string (esc_path, "");
+	g_free (esc_path);
 
 	search_data_set_start_from (data->search_data, full_path);
 	g_free (full_path);
@@ -285,18 +303,9 @@ search_clicked_cb (GtkWidget  *widget,
 
 	search_data_set_date (data->search_data, gnome_date_edit_get_time (GNOME_DATE_EDIT (data->s_date_dateedit)));
 
-	/* pop up the progress dialog. */
+	/**/
 
-	gtk_widget_hide (data->dialog);
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (data->p_notebook), 0);
-	gtk_widget_set_sensitive (data->p_searching_in_hbox, TRUE);
-	gtk_widget_set_sensitive (data->p_view_button, FALSE);
-	gtk_widget_set_sensitive (data->p_search_button, FALSE);
-	gtk_widget_set_sensitive (data->p_close_button, FALSE);
-	gtk_label_set_text (GTK_LABEL (data->p_images_label), "");
-	gtk_widget_show (data->search_progress_dialog);
-
-	search_images_async (data);
+	start_search_now (data);
 }
 
 
@@ -319,7 +328,11 @@ new_search_clicked_cb (GtkWidget *widget,
 		       DialogData *data)
 {
 	cancel_progress_dlg_cb (widget, data);
+
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	gtk_widget_hide (data->search_progress_dialog);
+
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
 	gtk_widget_show (data->dialog);
 }
 
@@ -329,7 +342,6 @@ static void
 view_result_cb (GtkWidget  *widget, 
 		DialogData *data)
 {
-	GThumbWindow *window = data->window;
 	Catalog      *catalog;
 	char         *catalog_name, *catalog_path, *catalog_name_utf8;
 	GList        *scan;
@@ -357,10 +369,10 @@ view_result_cb (GtkWidget  *widget,
 	if (! catalog_write_to_disk (catalog, &gerror)) 
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), &gerror);
 
+	gth_browser_go_to_catalog (data->browser, catalog_path);
+
 	gtk_widget_destroy (data->search_progress_dialog);
 	catalog_free (catalog);
-
-	window_go_to_catalog (window, catalog_path);
 	g_free (catalog_path);
 }
 
@@ -370,7 +382,6 @@ static void
 save_result_cb (GtkWidget  *widget, 
 		DialogData *data)
 {
-	GThumbWindow *window = data->window;
 	char         *catalog_path;
 	Catalog      *catalog;
 	GList        *scan;
@@ -389,10 +400,10 @@ save_result_cb (GtkWidget  *widget,
 	if (! catalog_write_to_disk (catalog, &gerror))
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), &gerror);
 
-	catalog_free (catalog);
-	gtk_widget_destroy (data->search_progress_dialog);
+	gth_browser_go_to_catalog (data->browser, catalog_path);
 
-	window_go_to_catalog (window, catalog_path);
+	gtk_widget_destroy (data->search_progress_dialog);
+	catalog_free (catalog);
 	g_free (catalog_path);
 }
 
@@ -513,18 +524,26 @@ response_cb (GtkWidget  *dialog,
 }
 
 
+static gboolean
+idle_start_search_cb (gpointer data) 
+{
+	start_search_now (data);
+	return FALSE;
+}
+
+
 static void
-dlg_search_ui (GThumbWindow *window, 
-	       char         *catalog_path, 
-	       gboolean      start_search)
+dlg_search_ui (GthBrowser *browser, 
+	       char       *catalog_path, 
+	       gboolean    start_search)
 {
 	DialogData        *data;
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
-	GValue             value = {0, };
+
+	g_return_if_fail (GTH_IS_BROWSER (browser));
 
 	data = g_new0 (DialogData, 1);
-
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" SEARCH_GLADE_FILE, NULL, NULL);
 	if (! data->gui) {
 		g_free (data);
@@ -538,7 +557,7 @@ dlg_search_ui (GThumbWindow *window,
 	data->keywords_patterns = NULL; 
 	data->dirs = NULL;
 	data->files = NULL;
-	data->window = window;
+	data->browser = browser;
 	data->handle = NULL;
 	data->search_data = NULL;
 	data->uri = NULL;
@@ -549,8 +568,7 @@ dlg_search_ui (GThumbWindow *window,
 
 	data->dialog = glade_xml_get_widget (data->gui, "search_dialog");
 
-	data->s_start_from_fileentry = glade_xml_get_widget (data->gui, "s_start_from_fileentry");
-	data->s_start_from_entry = glade_xml_get_widget (data->gui, "s_start_from_entry");
+	data->s_start_from_filechooserbutton = glade_xml_get_widget (data->gui, "s_start_from_filechooserbutton");
 	data->s_include_subfold_checkbutton = glade_xml_get_widget (data->gui, "s_include_subfold_checkbutton");
 	data->s_filename_entry = glade_xml_get_widget (data->gui, "s_filename_entry");
 	data->s_comment_entry = glade_xml_get_widget (data->gui, "s_comment_entry");
@@ -587,33 +605,43 @@ dlg_search_ui (GThumbWindow *window,
 
 	/* Set widgets data. */
 
-	/* Make use of the new filechooser */
-
-	g_value_init (&value, G_TYPE_BOOLEAN);
-	g_value_set_boolean (&value, TRUE);
-	g_object_set_property (G_OBJECT (data->s_start_from_fileentry),"use_filechooser", &value);
-	
-	/**/
-
 	if (catalog_path == NULL) {
-		if (data->window->dir_list->path != NULL)
-			_gtk_entry_set_filename_text (GTK_ENTRY (data->s_start_from_entry), data->window->dir_list->path);
-		else
-			_gtk_entry_set_filename_text (GTK_ENTRY (data->s_start_from_entry), g_get_home_dir ());
+		char *esc_uri = NULL;
+
+		esc_uri = gnome_vfs_escape_host_and_path_string (gth_browser_get_current_directory (data->browser));
+		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->s_start_from_filechooserbutton), esc_uri);
+		g_free (esc_uri);
+
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->s_include_subfold_checkbutton), eel_gconf_get_boolean (PREF_SEARCH_RECURSIVE, TRUE));
 
 	} else {
 		Catalog    *catalog;
 		SearchData *search_data;
+		char       *esc_uri = NULL;
 
 		catalog = catalog_new ();
 		catalog_load_from_disk (catalog, data->catalog_path, NULL);
 
 		search_data = catalog->search_data;
 
-		data->all_keywords =  search_data->all_keywords;
+		free_search_criteria_data (data);
+		search_data_free (data->search_data);
+		data->search_data = search_data_new ();
+		search_data_copy (data->search_data, search_data);
+		
+		data->all_keywords =  data->search_data->all_keywords;
+		data->file_patterns = search_util_get_file_patterns (data->search_data->file_pattern);
+		data->comment_patterns = search_util_get_patterns (data->search_data->comment_pattern);
+		data->place_patterns = search_util_get_patterns (data->search_data->place_pattern);
+		data->keywords_patterns = search_util_get_patterns (data->search_data->keywords_pattern);
 
-		_gtk_entry_set_filename_text (GTK_ENTRY (data->s_start_from_entry), search_data->start_from);
+		/**/
+		
+		esc_uri = gnome_vfs_escape_host_and_path_string (search_data->start_from);
+		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->s_start_from_filechooserbutton), esc_uri);
+		g_free (esc_uri);
+
+		/**/
 	
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->s_include_subfold_checkbutton), search_data->recursive);
 		
@@ -707,17 +735,18 @@ dlg_search_ui (GThumbWindow *window,
 
 	/* Run dialog. */
 
+	gtk_window_set_transient_for (GTK_WINDOW (data->search_progress_dialog), 
+				      GTK_WINDOW (browser));
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
-				      GTK_WINDOW (window->app));
+				      GTK_WINDOW (browser));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
-	gtk_window_set_transient_for (GTK_WINDOW (data->search_progress_dialog), GTK_WINDOW (window->app));
 
 	gtk_widget_grab_focus (data->s_filename_entry);
 
 	if (! start_search) 
 		gtk_widget_show (data->dialog);
 	else 
-		search_clicked_cb (NULL, data);
+		g_idle_add (idle_start_search_cb, data);
 }
 
 
@@ -728,7 +757,9 @@ static gboolean
 pattern_matched_by_keywords (char  *pattern,
 			     char **keywords)
 {
-	int i;
+	GPatternSpec *spec;
+	gboolean      retval = FALSE;
+	int           i;
 
 	if (pattern == NULL)
 		return TRUE;
@@ -736,11 +767,23 @@ pattern_matched_by_keywords (char  *pattern,
 	if ((keywords == NULL) || (keywords[0] == NULL))
 		return FALSE;
 
-	for (i = 0; keywords[i] != NULL; i++) 
-		if (g_utf8_fnmatch (pattern, keywords[i], FNM_CASEFOLD) == 0)
-			return TRUE;
+	spec = g_pattern_spec_new (pattern);
+	for (i = 0; keywords[i] != NULL; i++) {
+		char     *case_string;
+		gboolean  match;
 
-	return FALSE;
+		case_string = g_utf8_casefold (keywords[i], -1);
+		match = g_pattern_match_string (spec, case_string);
+		g_free (case_string);
+
+		if (match) {
+			retval = TRUE;
+			break;
+		}
+	}
+	g_pattern_spec_free (spec);
+
+	return retval;
 }
 
 
@@ -748,19 +791,25 @@ static gboolean
 match_patterns (char       **patterns, 
 		const char  *string)
 {
-	int i;
+	char     *case_string;
+	int       i;
+	gboolean  retval = FALSE;
 
-	if (patterns[0] == NULL)
+	if ((patterns == NULL) || (patterns[0] == NULL))
 		return TRUE;
 
 	if (string == NULL)
 		return FALSE;
 
+	case_string = g_utf8_casefold (string, -1);
 	for (i = 0; patterns[i] != NULL; i++) 
-		if (g_utf8_fnmatch (patterns[i], string, FNM_CASEFOLD) == 0)
-			return TRUE;
-	
-	return FALSE;
+		if (g_pattern_match_simple (patterns[i], case_string)) {
+			retval = TRUE;
+			break;
+		}
+	g_free (case_string);
+
+	return retval;
 }
 
 
@@ -777,7 +826,7 @@ load_parents_comments (DialogData *data,
 		g_free (tmp);
 
 		if (g_hash_table_lookup (data->folders_comment, parent) == NULL) {
-			CommentData *comment_data = comments_load_comment (parent);
+			CommentData *comment_data = comments_load_comment (parent, FALSE);
 			if (comment_data == NULL)
 				comment_data = comment_data_new ();
 
@@ -846,7 +895,7 @@ file_respects_search_criteria (DialogData *data,
 
 	load_parents_comments (data, filename);
 
-	comment_data = comments_load_comment (filename);
+	comment_data = comments_load_comment (filename, FALSE);
 	if (comment_data == NULL) 
 		comment_data = comment_data_new ();
 	add_parents_comments (comment_data, data, filename);
@@ -931,8 +980,8 @@ add_file_list (DialogData *data, GList *file_list)
 		filename = file_name_from_path (path);
 		folder = remove_level_from_path (path);
 
-		utf8_file = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
-		utf8_folder = g_filename_to_utf8 (folder, -1, NULL, NULL, NULL);
+		utf8_file = g_filename_display_name (filename);
+		utf8_folder = g_filename_display_name (folder);
 		
 		gtk_list_store_append (data->p_progress_tree_model, &iter);
 		gtk_list_store_set (data->p_progress_tree_model, 
@@ -1073,31 +1122,22 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 
 
 static void 
-search_dir_async (DialogData *data, char *dir)
+search_dir_async (DialogData *data, 
+		  char       *dir)
 {
-	char *escaped;
-	char *start_from;
-
-	start_from = g_strdup (dir);
-
 	_gtk_entry_set_filename_text (GTK_ENTRY (data->p_current_dir_entry), dir);
 
 	/**/
 
 	if (data->uri != NULL)
 		gnome_vfs_uri_unref (data->uri);
-
-	escaped = gnome_vfs_escape_host_and_path_string (start_from);
-	data->uri = gnome_vfs_uri_new (escaped);
-
-	g_free (start_from);
-	g_free (escaped);
+	data->uri = new_uri_from_path (dir);
 
 	gnome_vfs_async_load_directory_uri (
 		& (data->handle),
 		data->uri,
 		GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
-		32 /* items_per_notification FIXME */,
+		128 /* items_per_notification FIXME */,
 		GNOME_VFS_PRIORITY_DEFAULT,
 		directory_load_cb,
 		data);
@@ -1317,8 +1357,8 @@ static void
 choose_categories_dialog (DialogData *data)
 {
 	CategoriesDialogData *cdata;
-	GtkCellRenderer   *renderer;
-	GtkTreeViewColumn *column;
+	GtkCellRenderer      *renderer;
+	GtkTreeViewColumn    *column;
 
 	cdata = g_new (CategoriesDialogData, 1);
 	cdata->data = data;
