@@ -48,6 +48,7 @@
 #include "main.h"
 #include "preferences.h"
 #include "totem-scrsaver.h"
+#include "gs-fade.h"
 
 #include "icons/pixbufs.h"
 
@@ -84,12 +85,10 @@ struct _GthFullscreenPrivateData {
 
 	GtkWidget       *viewer;
 	GtkWidget       *toolbar_window;
-	GtkWidget       *info_button;
-	GtkWidget       *pause_button;
-	GtkWidget       *prev_button;
-	GtkWidget       *next_button;
 
 	TotemScrsaver   *screensaver;
+	GSFade          *fade;
+	gboolean         use_fade;
 
 	/* comment */
 
@@ -135,6 +134,9 @@ gth_fullscreen_finalize (GObject *object)
 
 		g_object_unref (priv->screensaver);
 
+		gs_fade_reset (priv->fade);
+		g_object_unref (priv->fade);
+
 		g_free (priv->image_path);
 		g_free (priv->requested_path);
 		path_list_free (priv->file_list);
@@ -173,8 +175,10 @@ slideshow_timeout_cb (gpointer data)
 		priv->slideshow_timeout = 0;
 	}
 
-	if (!priv->slideshow_paused)
+	if (!priv->slideshow_paused) {
+		priv->use_fade = TRUE;
 		load_next_image (fullscreen);
+	}
 
 	return FALSE;
 }
@@ -197,18 +201,39 @@ continue_slideshow (GthFullscreen *fullscreen)
 
 
 static void
-preloader_requested_error_cb (GThumbPreloader *gploader,
-			      GthFullscreen   *fullscreen)
+viewer_image_loaded_cb (ImageViewer   *iviewer,
+			GthFullscreen *fullscreen)
 {
 	GthFullscreenPrivateData *priv = fullscreen->priv;
 
-	image_viewer_set_void (IMAGE_VIEWER (priv->viewer));
-
+	if (priv->use_fade) {
+		gs_fade_in (priv->fade);
+		return;
+	}
+	
+	gs_fade_reset (priv->fade);
 	if (priv->slideshow)
 		continue_slideshow (fullscreen);
 }
 
 
+static void
+image_loaded (GthFullscreen *fullscreen)
+{
+	g_signal_emit_by_name (fullscreen->priv->viewer, "image_loaded", 0);
+}
+
+
+
+static void
+preloader_requested_error_cb (GThumbPreloader *gploader,
+			      GthFullscreen   *fullscreen)
+{
+	image_viewer_set_void (IMAGE_VIEWER (fullscreen->priv->viewer));
+	image_loaded (fullscreen);
+}
+
+
 static void
 preloader_requested_done_cb (GThumbPreloader *gploader,
 			     GthFullscreen   *fullscreen)
@@ -219,9 +244,31 @@ preloader_requested_done_cb (GThumbPreloader *gploader,
 	loader = gthumb_preloader_get_loader (priv->preloader, priv->requested_path);
 	if (loader != NULL) 
 		image_viewer_load_from_image_loader (IMAGE_VIEWER (priv->viewer), loader);
+	/*
+	image_loaded (fullscreen);
+	*/
+}
 
-	if (priv->slideshow)
-		continue_slideshow (fullscreen);
+
+static void real_load_current_image (GthFullscreen *fullscreen);
+
+
+static void
+fade_faded_cb (GSFade          *fade,
+	       GsFadeDirection  direction,
+	       GthFullscreen   *fullscreen)
+{
+	switch (direction) {
+	case GS_FADE_DIRECTION_OUT:
+		real_load_current_image (fullscreen);
+		break;
+		
+	case GS_FADE_DIRECTION_IN:
+		if (fullscreen->priv->slideshow)
+			continue_slideshow (fullscreen);
+		fullscreen->priv->use_fade = FALSE;
+		break;
+	}
 }
 
 
@@ -246,6 +293,13 @@ gth_fullscreen_init (GthFullscreen *fullscreen)
 
 	priv->screensaver = totem_scrsaver_new ();
 
+	priv->fade = gs_fade_new ();
+	g_signal_connect (G_OBJECT (priv->fade),
+			  "faded",
+			  G_CALLBACK (fade_faded_cb),
+			  fullscreen);
+	priv->use_fade = FALSE;
+
 	g_object_set (fullscreen, 
 		      "close_on_save", TRUE,
 		      NULL);
@@ -461,7 +515,7 @@ zoom_changed_cb (GtkWidget     *widget,
 
 
 static void
-load_current_image (GthFullscreen *fullscreen)
+real_load_current_image (GthFullscreen *fullscreen)
 {
 	GthFullscreenPrivateData *priv = fullscreen->priv;
 	gboolean from_pixbuf = TRUE;
@@ -471,7 +525,7 @@ load_current_image (GthFullscreen *fullscreen)
 
 		if ((priv->image != NULL) 
 		    && (priv->image_path != NULL) 
-		    && (strcmp (priv->image_path, filename) == 0))
+		    && same_uri (priv->image_path, filename))
 			image_viewer_set_pixbuf (IMAGE_VIEWER (priv->viewer), priv->image);
 
 		else {
@@ -492,8 +546,23 @@ load_current_image (GthFullscreen *fullscreen)
 
 	update_sensitivity (fullscreen);
 
-	if (priv->slideshow && from_pixbuf)
-		continue_slideshow (fullscreen);
+	if (from_pixbuf)
+		image_loaded (fullscreen);
+}
+
+
+static void
+load_current_image (GthFullscreen *fullscreen)
+{
+	GthFullscreenPrivateData *priv = fullscreen->priv;
+
+	if (priv->use_fade) {
+		gs_fade_out (fullscreen->priv->fade);
+		return;
+	}
+
+	gs_fade_reset (fullscreen->priv->fade);
+	real_load_current_image (fullscreen);
 }
 
 
@@ -532,7 +601,7 @@ load_first_or_last_image (GthFullscreen *fullscreen,
 		} else if (priv->image_path != NULL)
 			priv->current = g_list_find_custom (priv->file_list, 
 							    priv->image_path,
-							    (GCompareFunc) strcmp);
+							    (GCompareFunc) uricmp);
 		
 		if (priv->current == NULL)
 			priv->current = priv->file_list;
@@ -543,7 +612,7 @@ load_first_or_last_image (GthFullscreen *fullscreen,
 	    && (priv->slideshow_direction != GTH_DIRECTION_RANDOM)) 
 		priv->current = g_list_find_custom (priv->file_list, 
 						    priv->image_path,
-						    (GCompareFunc) strcmp);
+						    (GCompareFunc) uricmp);
 
 	load_current_image (fullscreen);
 }
@@ -568,7 +637,7 @@ load_next_image (GthFullscreen *fullscreen)
 {
 	GthFullscreenPrivateData *priv = fullscreen->priv;
 	GList *next;
-
+	
 	next = get_next_image (fullscreen);
 
 	if (next != NULL) {
@@ -743,7 +812,7 @@ pos_from_path (GList      *list,
 
 	for (scan = list; scan; scan = scan->next) {
 		char *l_path = scan->data;
-		if (strcmp (l_path, path) == 0)
+		if (same_uri (l_path, path))
 			return i;
 		i++;
 	}
@@ -806,17 +875,6 @@ get_file_info (GthFullscreen *fullscreen)
 }
 
 
-static void
-set_button_active_no_notify (gpointer      data,
-			     GtkWidget    *button,
-			     gboolean      active)
-{
-	g_signal_handlers_block_by_data (button, data);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), active);
-	g_signal_handlers_unblock_by_data (button, data);
-}
-
-
 static void
 show_comment_on_image (GthFullscreen *fullscreen)
 {
@@ -850,7 +908,6 @@ show_comment_on_image (GthFullscreen *fullscreen)
 		return;
 
 	priv->comment_visible = TRUE;
-	set_button_active_no_notify (NULL, priv->info_button, TRUE);
 	
 	comment = NULL;
 	if (cdata != NULL) {
@@ -975,7 +1032,6 @@ hide_comment_on_image (GthFullscreen *fullscreen)
 		return;
 
 	priv->comment_visible = FALSE;
-	set_button_active_no_notify (NULL, priv->info_button, FALSE);
 
 	gdk_draw_drawable (priv->viewer->window,
 			   priv->viewer->style->black_gc,
@@ -992,11 +1048,8 @@ _show_cursor__hide_comment (GthFullscreen *fullscreen)
 {
 	GthFullscreenPrivateData *priv = fullscreen->priv;
 
-	if (priv->slideshow) {
+	if (priv->slideshow) 
 		priv->slideshow_paused = TRUE;
-		if (priv->pause_button != NULL)
-			set_button_active_no_notify (NULL, priv->pause_button, TRUE);
-	}
 
 	if (priv->mouse_hide_id != 0)
 		g_source_remove (priv->mouse_hide_id);
@@ -1039,17 +1092,17 @@ viewer_key_press_cb (GtkWidget   *widget,
 	GthFullscreenPrivateData *priv = fullscreen->priv;
 	GthWindow     *window = (GthWindow*) fullscreen;
 	ImageViewer   *viewer = (ImageViewer*) fullscreen->priv->viewer;
-	gboolean       retval = FALSE;
-
+	gboolean       retval = TRUE;
+	GtkAction     *a;
 
 	if ((event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK) {
 		switch (event->keyval) {
 			/* Exit fullscreen mode. */
 		case GDK_w:
 			gth_window_close (window);
-			retval = TRUE;
 			break;
 		default:
+			retval = FALSE;
 			break;
 		}
 
@@ -1067,14 +1120,13 @@ viewer_key_press_cb (GtkWidget   *widget,
 	case GDK_f:
 	case GDK_F11:
 		gth_window_close (window);
-		retval = TRUE;
 		break;
 
 	case GDK_s:
-		if (fullscreen->priv->slideshow) {
+		if (fullscreen->priv->slideshow) 
 			gth_window_close (window);
-			retval = TRUE;
-		}
+		else
+			retval = FALSE;
 		break;
 
 		/* Zoom in. */
@@ -1130,7 +1182,6 @@ viewer_key_press_cb (GtkWidget   *widget,
 		break;
 
 		/* Load previous image. */
-	case GDK_p:
 	case GDK_b:
 	case GDK_BackSpace:
 	case GDK_Page_Up:
@@ -1151,6 +1202,12 @@ viewer_key_press_cb (GtkWidget   *widget,
 		load_current_image (fullscreen);
 		break;
 
+	case GDK_p:
+		a = gtk_ui_manager_get_action (priv->ui, "/ToolBar/View_PauseSlideshow");
+		if (a != NULL)
+			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (a), !priv->slideshow_paused);
+		break;
+
 		/* Edit comment. */
 	case GDK_c:
 		_show_cursor__hide_comment (fullscreen);
@@ -1174,6 +1231,36 @@ viewer_key_press_cb (GtkWidget   *widget,
 		}
 		break;
 
+		/* Rotate clockwise without saving */
+	case GDK_r: 
+		gth_window_activate_action_alter_image_rotate90 (NULL, window);
+		break;
+
+		/* Rotate counter-clockwise without saving */
+	case GDK_e:
+		gth_window_activate_action_alter_image_rotate90cc (NULL, window);
+		break;
+
+		/* Lossless clockwise rotation. */
+	case GDK_bracketright:
+		gth_window_activate_action_tools_jpeg_rotate_right (NULL, window);
+		break;
+
+		/* Lossless counter-clockwise rotation */
+	case GDK_bracketleft:
+		gth_window_activate_action_tools_jpeg_rotate_left (NULL, window);
+		break;
+
+		/* Flip image */
+	case GDK_l:
+		gth_window_activate_action_alter_image_flip (NULL, window);
+		break;
+
+		/* Mirror image */
+	case GDK_m:
+		gth_window_activate_action_alter_image_mirror (NULL, window);
+		break;
+
 		/* Delete selection. */
 	case GDK_Delete: 
 	case GDK_KP_Delete:
@@ -1184,6 +1271,7 @@ viewer_key_press_cb (GtkWidget   *widget,
 	default:
 		if (priv->comment_visible) 
 			hide_comment_on_image (fullscreen);
+		retval = FALSE;
 		break;
 	}
 
@@ -1280,7 +1368,7 @@ delete_list_from_file_list (GthFullscreen *fullscreen,
 		
 		deleted = g_list_find_custom (fullscreen->priv->file_list, 
 					      filename,
-					      (GCompareFunc) strcmp);
+					      (GCompareFunc) uricmp);
 		if (deleted != NULL) {
 			if (fullscreen->priv->current == deleted) {
 				reload_current_image = TRUE;
@@ -1316,10 +1404,19 @@ monitor_update_files_cb (GthMonitor      *monitor,
 	switch (event) {
 	case GTH_MONITOR_EVENT_CREATED:
 	case GTH_MONITOR_EVENT_CHANGED:
+		if ((fullscreen->priv->image_path != NULL) 
+		    && (g_list_find_custom (list, 
+					    fullscreen->priv->image_path,
+					    (GCompareFunc) uricmp) != NULL)) {
+			g_free (fullscreen->priv->image_path);
+			fullscreen->priv->image_path = NULL;
+			if (fullscreen->priv->image != NULL)
+				g_object_unref (fullscreen->priv->image);
+		}
 		if ((fullscreen->priv->current != NULL) 
 		    && (g_list_find_custom (list, 
 					    fullscreen->priv->current->data,
-					    (GCompareFunc) strcmp) == NULL))
+					    (GCompareFunc) uricmp) != NULL))
 			load_current_image (fullscreen);
 		break;
 
@@ -1343,7 +1440,7 @@ monitor_file_renamed_cb (GthMonitor    *monitor,
 
 	renamed_image = g_list_find_custom (fullscreen->priv->file_list,
 					    old_name,
-					    (GCompareFunc) strcmp);
+					    (GCompareFunc) uricmp);
 
 	if (renamed_image == NULL)
 		return;
@@ -1354,7 +1451,7 @@ monitor_file_renamed_cb (GthMonitor    *monitor,
 	if (fullscreen->priv->image_path == NULL)
 		return;
 
-	if (strcmp (old_name, fullscreen->priv->image_path) != 0)
+	if (! same_uri (old_name, fullscreen->priv->image_path))
 		return;
 
 	g_free (fullscreen->priv->image_path);
@@ -1434,6 +1531,10 @@ gth_fullscreen_construct (GthFullscreen *fullscreen,
 			  "zoom_changed",
 			  G_CALLBACK (zoom_changed_cb), 
 			  fullscreen);
+	g_signal_connect (G_OBJECT (priv->viewer),
+			  "image_loaded",
+			  G_CALLBACK (viewer_image_loaded_cb),
+			  fullscreen);
 
 	gtk_widget_show (priv->viewer);
 	gnome_app_set_contents (GNOME_APP (fullscreen), priv->viewer);
@@ -1520,6 +1621,8 @@ create_toolbar_window (GthFullscreen *fullscreen)
 	/**/
 
 	priv->toolbar_window = gtk_window_new (GTK_WINDOW_POPUP);
+	gtk_window_set_screen (GTK_WINDOW (priv->toolbar_window),
+			       gtk_widget_get_screen (priv->viewer));
 	gtk_window_set_default_size (GTK_WINDOW (priv->toolbar_window),
 				     gdk_screen_get_width (gtk_widget_get_screen (priv->toolbar_window)),
 				     -1);
